"""Broadcast Manager Module."""

from masonite.contracts import BroadcastManagerContract
from masonite.managers import Manager


class BroadcastManager(Manager, BroadcastManagerContract):
    """Manages all broadcast drivers.

    Arguments:
        Manager {masonite.managers.Manager} -- The base Manager class.
    """

    config = 'BroadcastConfig'
    driver_prefix = 'Broadcast'


class Broadcast:
    """Dummy class that will be used to swap out the manager in the container."""

    pass



"""Starts Interactive Console Command."""
import code
import sys

from cleo import Command

BANNER = """Masonite Python {} Console
This interactive console has the following things imported:
    container as 'app'

Type `exit()` to exit."""


class TinkerCommand(Command):
    """
    Run a python shell with the container pre-loaded.

    tinker
    """

    def handle(self):
        from wsgi import container

        version = "{}.{}.{}".format(
            sys.version_info.major,
            sys.version_info.minor,
            sys.version_info.micro
        )
        banner = BANNER.format(version)

        code.interact(banner=banner, local={"app": container})



"""Maintainance Mode Middleware."""
import os
from masonite.request import Request
from config import application


class MaintenanceModeMiddleware:

    def __init__(self, request: Request):
        self.request = request

    def before(self):
        down = os.path.exists(os.path.join(application.BASE_DIRECTORY, 'bootstrap/down'))
        if down is True:
            self.request.status(503)



from masonite.testsuite import TestSuite, generate_wsgi
from masonite.testing import BaseRequest


class MockJson(BaseRequest):

    def __init__(self, url, container):
        self.url = url
        self.container = container

    def json(self, request_method, url, data):
        wsgi = generate_wsgi()
        wsgi['PATH_INFO'] = url
        wsgi['CONTENT_TYPE'] = 'application/json'

    def contains(self, value):
        return value in self.container.make('Response')

    def _run_container(self, wsgi):
        return TestSuite().create_container(wsgi, container=self.container)

    def _bind_user_to_request(self, request, container):
        request.set_user(self._user)
        return self



"""Module for the Routing System."""

import cgi
import importlib
import json

from masonite.exceptions import RouteMiddlewareNotFound, InvalidRouteCompileException, RouteException
from masonite.view import View


class Route:
    """Route class used to handle routing."""

    route_compilers = {
        'int': r'(\d+)',
        'integer': r'(\d+)',
        'string': r'([a-zA-Z]+)',
        'default': r'([\w.-]+)',
        'signed': r'([\w\-=]+)'
    }

    def __init__(self, environ=None):
        """Route constructor.

        Keyword Arguments:
            environ {dict} -- WSGI environ (default: {None})
        """
        self.url_list = []

        if environ:
            self.environ = environ
            self.url = environ['PATH_INFO']

            if self.is_not_get_request():
                self.environ['QUERY_STRING'] = self.set_post_params()

    def load_environ(self, environ):
        """Load the WSGI environ into the class.

        Arguments:
            environ {dict} -- WSGI environ

        Returns:
            self
        """
        self.environ = environ
        self.url = environ['PATH_INFO']

        if self.is_not_get_request():
            self.environ['QUERY_STRING'] = self.set_post_params()

        return self

    def get(self, route, output=None):
        """Return the output.

        Arguments:
            route {masonite.routes.BaseHttpRoute} -- The current route being executed.

        Keyword Arguments:
            output {string} -- Returns the output (default: {None})

        Returns:
            string
        """
        return output

    def set_post_params(self):
        """Return the correct input.

        Returns:
            dict -- Dictionary of post parameters.
        """
        fields = None
        if self.is_not_get_request():
            if 'CONTENT_TYPE' in self.environ and 'application/json' in self.environ['CONTENT_TYPE']:
                try:
                    request_body_size = int(
                        self.environ.get('CONTENT_LENGTH', 0))
                except (ValueError):
                    request_body_size = 0

                request_body = self.environ['wsgi.input'].read(
                    request_body_size)
                return json.loads(request_body)
            else:
                fields = cgi.FieldStorage(
                    fp=self.environ['wsgi.input'], environ=self.environ, keep_blank_values=1)
                return fields

    def is_post(self):
        """Check to see if the current request is a POST request.

        Returns:
            bool
        """
        if self.environ['REQUEST_METHOD'] == 'POST':
            return True

        return False

    def is_not_get_request(self):
        """Check if current request is not a get request.

        Returns:
            bool
        """
        if not self.environ['REQUEST_METHOD'] == 'GET':
            return True

        return False

    def compile(self, key, to=''):
        self.route_compilers.update({key: to})
        return self

    def generated_url_list(self):
        """Return the URL list.

        Returns:
            list -- URL list.
        """
        return self.url_list


class BaseHttpRoute:
    """Base route for HTTP routes."""

    method_type = 'GET'
    output = False
    route_url = None
    request = None
    named_route = None
    required_domain = None
    module_location = 'app.http.controllers'
    list_middleware = None

    def route(self, route, output):
        """Load the route into the class. This also looks for the controller and attaches it to the route.

        Arguments:
            route {string} -- This is a URI to attach to the route (/dashboard/user).
            output {string|object} -- Controller to attach to the route.

        Returns:
            self
        """
        self._find_controller(output)
        self.route_url = route
        return self

    def view(self, route, template, dictionary={}):
        view_route = ViewRoute(self.method_type, route, template, dictionary)
        return view_route

    def _find_controller(self, controller):
        """Find the controller to attach to the route.

        Arguments:
            controller {string|object} -- String or object controller to search for.

        Returns:
            None
        """
        # If the output specified is a string controller
        if isinstance(controller, str):
            mod = controller.split('@')
            # If trying to get an absolute path via a string
            if mod[0].startswith('/'):
                self.module_location = '.'.join(
                    mod[0].replace('/', '').split('.')[0:-1])
            elif '.' in mod[0]:
                # This is a deeper module controller
                self.module_location = self.module_location + '.' + '.'.join(mod[0].split('.')[:-1])
        else:
            if controller is None:
                return None

            fully_qualified_name = controller.__qualname__
            mod = fully_qualified_name.split('.')
            self.module_location = controller.__module__

        # Gets the controller name from the output parameter
        # This is used to add support for additional modules
        # like 'LoginController' and 'Auth.LoginController'
        get_controller = mod[0].split('.')[-1]

        try:
            # Import the module
            if isinstance(controller, str):
                module = importlib.import_module(
                    '{0}.'.format(self.module_location) + get_controller)
            else:
                module = importlib.import_module(
                    '{0}'.format(self.module_location))

            # Get the controller from the module
            self.controller = getattr(module, get_controller)

            # Set the controller method on class. This is a string
            self.controller_method = mod[1]

        except Exception as e:
            print('\033[93mWarning in routes/web.py!', e, '\033[0m')

    def get_response(self):
        # Resolve Controller Constructor
        controller = self.request.app().resolve(self.controller)

        # Resolve Controller Method
        response = self.request.app().resolve(
            getattr(controller, self.controller_method))

        if isinstance(response, View):
            response = response.rendered_template

        return response

    def domain(self, domain):
        """Set the subdomain for the route.

        Arguments:
            domain {string|list|tuple} -- The string or list of subdomains to attach to this route.

        Returns:
            self
        """
        self.required_domain = domain
        return self

    def module(self, module):
        """DEPRECATED :: The base module to look for string controllers.

        Arguments:
            module {string} -- The string representation of a module to look for controllers.

        Returns:
            self
        """
        self.module_location = module
        return self

    def has_required_domain(self):
        """Check if the route has the required subdomain before executing the route.

        Returns:
            bool
        """
        if self.request.has_subdomain() and (self.required_domain is '*' or self.request.subdomain == self.required_domain):
            return True
        return False

    def name(self, name):
        """Specify the name of the route.

        Arguments:
            name {string} -- Sets a name for the route.

        Returns:
            self
        """
        self.named_route = name
        return self

    def load_request(self, request):
        """Load the request into this class.

        Arguments:
            request {masonite.request.Request} -- Request object.

        Returns:
            self
        """
        self.request = request
        return self

    def middleware(self, *args):
        """Load a list of middleware to run.

        Returns:
            self
        """
        for arg in args:
            if arg not in self.list_middleware:
                self.list_middleware.append(arg)

        return self

    def run_middleware(self, type_of_middleware):
        """Run route middleware.

        Arguments:
            type_of_middleware {string} -- Type of middleware to be ran (before|after)

        Raises:
            RouteMiddlewareNotFound -- Thrown when the middleware could not be found.
        """
        # Get the list of middleware to run for a route.
        for arg in self.list_middleware:
            middleware_to_run = self.request.app().make('RouteMiddleware')[arg]
            if not isinstance(middleware_to_run, list):
                middleware_to_run = [middleware_to_run]

            try:
                for middleware in middleware_to_run:
                    located_middleware = self.request.app().resolve(middleware)
                    if hasattr(located_middleware, type_of_middleware):
                        getattr(located_middleware, type_of_middleware)()
            except KeyError:
                raise RouteMiddlewareNotFound(
                    "Could not find the '{0}' route middleware".format(arg))

    def compile_route_to_regex(self, router):
        """Compile the given route to a regex string.

        Arguments:
            route {string} -- URI of the route to compile.

        Returns:
            string -- Compiled URI string.
        """
        # Split the route
        split_given_route = self.route_url.split('/')
        # compile the provided url into regex
        url_list = []
        regex = '^'
        for regex_route in split_given_route:
            if '@' in regex_route:
                if ':' in regex_route:
                    try:
                        regex += router.route_compilers[regex_route.split(':')[
                            1]]
                    except KeyError:
                        raise InvalidRouteCompileException(
                            'Route compiler "{}" is not an available route compiler. '
                            'Verify you spelled it correctly or that you have added it using the compile() method.'.format(
                                regex_route.split(':')[1])
                        )
                else:
                    regex += router.route_compilers['default']

                regex += r'\/'

                # append the variable name passed @(variable):int to a list
                url_list.append(
                    regex_route.replace('@', '').split(':')[0]
                )
            else:
                regex += regex_route + r'\/'

        router.url_list = url_list
        regex += '$'
        return regex


class Get(BaseHttpRoute):
    """Class for specifying GET requests."""

    def __init__(self):
        """Get constructor."""
        self.method_type = ['GET']
        self.list_middleware = []


class Post(BaseHttpRoute):
    """Class for specifying POST requests."""

    def __init__(self):
        """Post constructor."""
        self.method_type = ['POST']
        self.list_middleware = []


class Match(BaseHttpRoute):
    """Class for specifying POST requests."""

    def __init__(self, method_type=['GET']):
        """Post constructor."""
        if not isinstance(method_type, list):
            raise RouteException("Method type needs to be a list. Got '{}'".format(method_type))

        # Make all method types in list uppercase
        self.method_type = [x.upper() for x in method_type]
        self.list_middleware = []


class Put(BaseHttpRoute):
    """Class for specifying PUT requests."""

    def __init__(self):
        """Put constructor."""
        self.method_type = ['PUT']
        self.list_middleware = []


class Patch(BaseHttpRoute):
    """Class for specifying Patch requests."""

    def __init__(self):
        """Patch constructor."""
        self.method_type = ['PATCH']
        self.list_middleware = []


class Delete(BaseHttpRoute):
    """Class for specifying Delete requests."""

    def __init__(self):
        """Delete constructor."""
        self.method_type = ['DELETE']
        self.list_middleware = []


class ViewRoute(BaseHttpRoute):

    def __init__(self, method_type, route, template, dictionary):
        """Class used for view routes.

        This class should be returned when a view is called on an HTTP route.
        This is useful when returning a view that doesn't need any special logic and only needs a dictionary.

        Arguments:
            method_type {string} -- The method type (GET, POST, PUT etc)
            route {string} -- The current route (/test/url)
            template {string} -- The template to use (dashboard/user)
            dictionary {dict} -- The dictionary to use to render the template.
        """
        self.list_middleware = []
        self.method_type = method_type
        self.route_url = route
        self.template = template
        self.dictionary = dictionary

    def get_response(self):
        return self.request.app().make('ViewClass').render(self.template, self.dictionary).rendered_template


class RouteGroup():
    """Class for specifying Route Groups."""

    def __new__(self, routes=[], middleware=[], domain=[], prefix='', name='', add_methods=[]):
        """Call when this class is first called. This is to give the ability to return a value in the constructor.

        Keyword Arguments:
            routes {list} -- List of routes. (default: {[]})
            middleware {list} -- List of middleware. (default: {[]})
            domain {list} -- String or list of domains to attach to all the routes. (default: {[]})
            prefix {str} -- Prefix to attach to all the route URI's. (default: {''})
            name {str} -- Base name to attach to all the routes. (default: {''})

        Returns:
            list -- Returns a list of routes.
        """
        from masonite.helpers.routes import flatten_routes
        self.routes = flatten_routes(routes)

        if middleware:
            self._middleware(self, *middleware)

        if add_methods:
            self._add_methods(self, *add_methods)

        if domain:
            self._domain(self, domain)

        if prefix:
            self._prefix(self, prefix)

        if name:
            self._name(self, name)

        return self.routes

    def _middleware(self, *middleware):
        """Attach middleware to all routes.

        Returns:
            list -- Returns list of routes.
        """
        for route in self.routes:
            route.middleware(*middleware)

        return self.routes

    def _add_methods(self, *methods):
        """Attach more methods to all routes.

        Returns:
            list -- Returns list of routes.
        """
        for route in self.routes:
            route.method_type.append(*methods)

        return self.routes

    def _domain(self, domain):
        """Attach a domain to all routes.

        Arguments:
            domain {str|list|tuple} -- List of domains to attach to all the routes.
        """
        for route in self.routes:
            route.domain(domain)

    def _prefix(self, prefix):
        """Prefix a string to all domain URI's.

        Arguments:
            prefix {str} -- String to prefix to all Routes.
        """
        for route in self.routes:
            route.route_url = prefix + route.route_url

    def _name(self, name):
        """Name to prefix to all routes.

        Arguments:
            name {str} -- String to prefix to all routes.
        """
        for route in self.routes:
            if isinstance(route.named_route, str):
                route.named_route = name + route.named_route



"""A RedirectionProvider Service Provider."""

from config import queue
from masonite.drivers import QueueAsyncDriver, QueueAmqpDriver
from masonite.managers import QueueManager
from masonite.provider import ServiceProvider
from masonite import Queue


class QueueProvider(ServiceProvider):

    wsgi = False

    def register(self):
        self.app.bind('QueueAsyncDriver', QueueAsyncDriver)
        self.app.bind('QueueAmqpDriver', QueueAmqpDriver)
        self.app.bind('QueueManager', QueueManager)
        self.app.bind('QueueConfig', queue)

    def boot(self, queue: QueueManager):
        self.app.bind('Queue', queue(self.app).driver(self.app.make('QueueConfig').DRIVER))
        self.app.swap(Queue, queue(self.app).driver(self.app.make('QueueConfig').DRIVER))



"""New Providers Command."""
from masonite.commands import BaseScaffoldCommand


class ProviderCommand(BaseScaffoldCommand):
    """
    Creates a new Service Provider.

    provider
        {name : Name of the Service Provider you want to create}
    """

    scaffold_name = 'Service Provider'
    base_directory = 'app/providers/'
    template = '/masonite/snippets/scaffold/provider'



"""Mail Manager Module."""

from masonite.contracts import MailManagerContract
from masonite.managers import Manager


class MailManager(Manager, MailManagerContract):
    """Manages all mail drivers.

    Arguments:
        Manager {masonite.managers.Manager} -- The base Manager class.
    """

    config = 'MailConfig'
    driver_prefix = 'Mail'

    def helper(self):
        """Helper Method to work with returning the driver from the MailManager.

        Returns:
            Mail Driver
        """
        return self.driver(self.container.make('MailConfig').DRIVER)


class Mail:
    """Dummy class that will be used to swap out the manager in the container."""

    pass



from masonite.testsuite import TestSuite, generate_wsgi
from masonite.testing import MockRoute, MockRequest, MockJson
import json
import io
# from masonite.testing.MockJson import MockJson


class UnitTest:

    def setup_method(self):
        self.container = TestSuite().create_container().container

    def controller(self):
        pass

    def get(self, url):
        wsgi = generate_wsgi()
        wsgi['PATH_INFO'] = url
        self.container = TestSuite().create_container(wsgi=wsgi).container
        return MockRequest(url, self.container)

    def route(self, url, method=False):
        for route in self.container.make('WebRoutes'):
            if route.route_url == url and (method in route.method_type or not method):
                return MockRoute(route, self.container)

    def routes(self, routes):
        self.container.bind('WebRoutes', self.container.make('WebRoutes') + routes)

    def json(self, url, data, method=['POST']):
        wsgi = generate_wsgi()
        wsgi['PATH_INFO'] = url
        wsgi['CONTENT_TYPE'] = 'application/json'
        wsgi['REQUEST_METHOD'] = method
        wsgi['CONTENT_LENGTH'] = len(str(json.dumps(data)))
        wsgi['wsgi.input'] = io.StringIO(json.dumps(data))
        self.container = TestSuite().create_container(wsgi=wsgi).container
        return MockJson(url, self.container)



"""Authentication Class."""

import uuid

import bcrypt


class Auth:
    """This class will be used to authenticate users based on the config/auth.py file."""

    _once = False

    def __init__(self, request, auth_model=None):
        """Auth constructor.

        Arguments:
            request {masonite.request.Request} -- The Request object.

        Keyword Arguments:
            auth_model {object} -- The model you want to authenticate with (default: {None})
        """
        self.request = request

        if auth_model:
            self.auth_model = auth_model
        else:
            from config import auth
            self.auth_model = auth.AUTH['model']

    def user(self):
        """Get the currently logged in user.

        Raises:
            exception -- Raised when there has been an error handling the user model.

        Returns:
            object|bool -- Returns the current authenticated user object or False or None if there is none.
        """
        try:
            if self.request.get_cookie('token'):
                return self.auth_model.where(
                    'remember_token', self.request.get_cookie('token')
                ).first()

            return False
        except Exception as exception:
            raise exception

        return None

    def login(self, name, password):
        """Login the user based on the parameters provided.

        Arguments:
            name {string} -- The field to authenticate. This could be a username or email address.
            password {string} -- The password to authenticate with.

        Raises:
            exception -- Raised when there has been an error handling the user model.

        Returns:
            object|bool -- Returns the current authenticated user object or False or None if there is none.
        """
        auth_column = self.auth_model.__auth__
        try:
            model = self.auth_model.where(auth_column, name).first()

            if model and bcrypt.checkpw(bytes(password, 'utf-8'), bytes(model.password, 'utf-8')):
                if not self._once:
                    remember_token = str(uuid.uuid4())
                    model.remember_token = remember_token
                    model.save()
                    self.request.cookie('token', remember_token)
                return model

        except Exception as exception:
            raise exception

        return False

    def logout(self):
        """Logout the current authenticated user.

        Returns:
            self
        """
        self.request.delete_cookie('token')
        return self

    def login_by_id(self, id):
        """Login a user by the user ID.

        Arguments:
            id {string|int} -- The ID of the user model record.

        Returns:
            object|False -- Returns the current authenticated user object or False or None if there is none.
        """
        model = self.auth_model.find(id)

        if model:
            if not self._once:
                remember_token = str(uuid.uuid4())
                model.remember_token = remember_token
                model.save()
                self.request.cookie('token', remember_token)
            return model

        return False

    def once(self):
        """Log in the user without saving a cookie.

        Returns:
            self
        """
        self._once = True
        return self



"""Base mail driver module."""

from masonite.drivers.BaseDriver import BaseDriver
from masonite.view import View
from masonite.app import App


class BaseMailDriver(BaseDriver):
    """Base mail driver class. This class is inherited by all mail drivers."""

    def __init__(self, app: App, view: View):
        """Base mail driver constructor.

        Arguments:
            MailConfig {module} -- This is the config.mail module.
            View {object} -- This is the masonite.view.View class.
        """
        self.config = app.make('MailConfig')
        self.to_address = None
        self.from_address = self.config.FROM
        self.message_subject = 'Subject'
        self.message_body = None
        self.view = view
        self._queue = False

    def to(self, user_email):
        """Set the user email address who you want to send mail to.

        Arguments:
            user_email {string} -- The user email address.

        Returns:
            self
        """
        if callable(user_email):
            user_email = user_email.email

        self.to_address = user_email
        return self

    def queue(self):
        """Set the user email address who you want to send mail to.

        Arguments:
            user_email {string} -- The user email address.

        Returns:
            self
        """
        self._queue = True
        return self

    def template(self, template_name, dictionary={}):
        """Create an email from a normal Jinja template.

        Arguments:
            template_name {string} -- The name of the template.

        Keyword Arguments:
            dictionary {dict} -- The data to be passed to the template. (default: {{}})

        Returns:
            self
        """
        self.message_body = self.view.render(template_name, dictionary).rendered_template
        return self

    def send_from(self, address):
        """Set the from address of who the sender should be.

        Arguments:
            address {string} -- A name used as the From field in an email.

        Returns:
            self
        """
        self.from_address = address
        return self

    def subject(self, subject):
        """Set the subject of an email.

        Arguments:
            subject {string} -- The subject of the email

        Returns:
            self
        """
        self.message_subject = subject
        return self



"""A StatusProvider Service Provider."""

from config import application
from masonite.response import Response
from masonite.provider import ServiceProvider


class ServerErrorExceptionHook:

    def load(self, app):
        if application.DEBUG:
            return

        request = app.make('Request')

        request.status(500)
        if app.make('ViewClass').exists('errors/500'):
            rendered_view = app.make('View')('errors/500')
        else:
            rendered_view = app.make('View')(
                '/masonite/snippets/statuscode', {'code': '500 Internal Server Error'})

        request.app().make(Response).view(rendered_view)


class StatusCodeProvider(ServiceProvider):

    def register(self):
        self.app.bind('ServiceErrorExceptionHook', ServerErrorExceptionHook())

    def boot(self):
        request = self.app.make('Request')
        response = self.app.make(Response)
        if request.is_status(200):
            return

        if request.get_status() not in (200, 301, 302):
            if self.app.make('ViewClass').exists('errors/{}'.format(request.get_status())):
                rendered_view = self.app.make('View')(
                    'errors/{}'.format(request.get_status()))
            else:
                rendered_view = self.app.make('View')('/masonite/snippets/statuscode', {
                    'code': request.get_status_code()
                })

            response.view(rendered_view, status=request.get_status())



from .BaseDriver import BaseDriver
from .BaseMailDriver import BaseMailDriver
from .BaseUploadDriver import BaseUploadDriver
from .BaseCacheDriver import BaseCacheDriver
from .BroadcastAblyDriver import BroadcastAblyDriver
from .BroadcastPusherDriver import BroadcastPusherDriver
from .CacheDiskDriver import CacheDiskDriver
from .CacheRedisDriver import CacheRedisDriver
from .MailMailgunDriver import MailMailgunDriver
from .MailSmtpDriver import MailSmtpDriver
from .MailLogDriver import MailLogDriver
from .MailTerminalDriver import MailTerminalDriver
from .QueueAsyncDriver import QueueAsyncDriver
from .QueueAmqpDriver import QueueAmqpDriver
from .SessionCookieDriver import SessionCookieDriver
from .SessionMemoryDriver import SessionMemoryDriver
from .UploadDiskDriver import UploadDiskDriver
from .UploadS3Driver import UploadS3Driver



class DriverNotFound(Exception):
    pass


class DriverLibraryNotFound(Exception):
    pass


class FileTypeException(Exception):
    pass


class RequiredContainerBindingNotFound(Exception):
    pass


class MissingContainerBindingNotFound(Exception):
    pass


class UnacceptableDriverType(Exception):
    pass


class ContainerError(Exception):
    pass


class InvalidCSRFToken(Exception):
    pass


class InvalidHTTPStatusCode(Exception):
    pass


class RouteMiddlewareNotFound(Exception):
    pass


class ResponseError(Exception):
    pass


class InvalidAutoloadPath(Exception):
    pass


class AutoloadContainerOverwrite(Exception):
    pass


class InvalidSecretKey(Exception):
    pass


class StrictContainerException(Exception):
    pass


class InvalidRouteCompileException(Exception):
    pass


class RouteException(Exception):
    pass


class DebugException(Exception):
    pass


class DumpException(Exception):
    pass



""" A QueueWorkCommand Command """

import inspect
import pickle

from cleo import Command

from config import queue
from masonite.exceptions import DriverLibraryNotFound


def callback(ch, method, properties, body):
    from wsgi import container
    job = pickle.loads(body)
    obj = job['obj']
    args = job['args']
    callback = job['callback']
    if inspect.isclass(obj):
        obj = container.resolve(obj)

    try:
        getattr(obj, callback)(*args)
    except AttributeError:
        obj(*args)

    ch.basic_ack(delivery_tag=method.delivery_tag)


class QueueWorkCommand(Command):
    """
    Start the queue worker

    queue:work
        {--c|channel=default : The channel to listen on the queue}
        {--f|fair : Send jobs to queues that have no jobs instead of randomly selecting a queue}
    """

    def handle(self):
        try:
            import pika
        except ImportError:
            raise DriverLibraryNotFound(
                "Could not find the 'pika' library. Run pip install pika to fix this.")

        connection = pika.BlockingConnection(pika.URLParameters('amqp://{}:{}@{}{}/{}'.format(
            queue.DRIVERS['amqp']['username'],
            queue.DRIVERS['amqp']['password'],
            queue.DRIVERS['amqp']['host'],
            ':' + str(queue.DRIVERS['amqp']['port']) if 'port' in queue.DRIVERS['amqp'] and queue.DRIVERS['amqp']['port'] else '',
            queue.DRIVERS['amqp']['vhost'] if 'vhost' in queue.DRIVERS['amqp'] and queue.DRIVERS['amqp']['vhost'] else '%2F'
        )))
        channel = connection.channel()

        channel.queue_declare(queue=self.option('channel'), durable=True)

        channel.basic_consume(callback,
                              queue=self.option('channel'))
        if self.option('fair'):
            channel.basic_qos(prefetch_count=1)

        self.info(' [*] Waiting to process jobs on the "{}" channel. To exit press CTRL+C'.format(
            self.option('channel')))
        channel.start_consuming()



"""Base cache driver module.
"""

from masonite.drivers.BaseDriver import BaseDriver


class BaseCacheDriver(BaseDriver):
    """Base class that all cache drivers inherit from.
    """

    def calculate_time(self, cache_type, cache_time):
        """Convert time to required unit
        Returns:
            self
        """

        cache_type = cache_type.lower()
        calc = 0

        if cache_type in ("second", "seconds"):
            # Set time now for
            calc = 1
        elif cache_type in ("minute", "minutes"):
            calc = 60
        elif cache_type in ("hour", "hours"):
            calc = 60 * 60
        elif cache_type in ("day", "days"):
            calc = 60 * 60 * 60
        elif cache_type in ("month", "months"):
            calc = 60 * 60 * 60 * 60
        elif cache_type in ("year", "years"):
            calc = 60 * 60 * 60 * 60 * 60
        else:
            raise ValueError(
                '{0} is not a valid caching type.'.format(cache_type))

        return cache_time * calc



"""A RedirectionProvider Service Provider."""

from config import broadcast
from masonite.drivers import BroadcastAblyDriver, BroadcastPusherDriver
from masonite.managers import BroadcastManager
from masonite.provider import ServiceProvider
from masonite import Broadcast


class BroadcastProvider(ServiceProvider):

    wsgi = False

    def register(self):
        self.app.bind('BroadcastConfig', broadcast)
        self.app.bind('BroadcastPusherDriver', BroadcastPusherDriver)
        self.app.bind('BroadcastAblyDriver', BroadcastAblyDriver)
        self.app.bind('BroadcastManager', BroadcastManager(self.app))

    def boot(self, broadcast: BroadcastManager):
        self.app.bind('Broadcast', self.app.make('BroadcastManager').driver(self.app.make('BroadcastConfig').DRIVER))
        self.app.swap(Broadcast, self.app.make('BroadcastManager').driver(self.app.make('BroadcastConfig').DRIVER))



"""Async Driver Method."""

import threading
import inspect

from masonite.contracts import QueueContract
from masonite.drivers import BaseDriver
from masonite.app import App


class QueueAsyncDriver(QueueContract, BaseDriver):
    """Queue Aysnc Driver."""

    def __init__(self, app: App):
        """Queue Async Driver.

        Arguments:
            Container {masonite.app.App} -- The application container.
        """
        self.container = app

    def push(self, *objects, args=(), callback='handle'):
        """Push objects onto the async stack.

        Arguments:
            objects {*args of objects} - This can be several objects as parameters into this method.
        """
        for obj in objects:
            if inspect.isclass(obj):
                obj = self.container.resolve(obj)

            try:
                thread = threading.Thread(
                    target=getattr(obj, callback), args=args, kwargs={})
            except AttributeError:
                # Could be wanting to call only a method asyncronously
                thread = threading.Thread(
                    target=obj, args=args, kwargs={})

            thread.start()



"""Module for the Service Provider."""

from masonite.helpers import random_string


class ServiceProvider:
    """Service provider class. Used as mediator for loading objects or entire features into the container."""

    wsgi = True

    def __init__(self):
        """Service provider constructor."""
        self.app = None

    def boot(self):
        """Use to boot things into the container. Typically ran after the register method has been ran."""
        pass

    def register(self):
        """Use to register objects into the container."""
        pass

    def load_app(self, app):
        """Load the container into the service provider.

        Arguments:
            app {masonite.app.App} -- Container object.

        Returns:
            self
        """
        self.app = app
        return self

    def routes(self, routes):
        """Add routes to the container.

        Arguments:
            routes {list} -- List of routes to add to the container
        """
        web_routes = self.app.make('WebRoutes')
        web_routes += routes

    def http_middleware(self, middleware):
        """Add HTTP middleware to the container.

        Arguments:
            middleware {list} -- List of middleware to add
        """
        http_middleware = self.app.make('HttpMiddleware')
        http_middleware += middleware

    def route_middleware(self, middleware):
        """Add route middleware to the container.

        Arguments:
            middleware {dict} -- A dictionary of route middleware to add
        """
        route_middleware = self.app.make('RouteMiddleware')
        route_middleware.update(middleware)

    def migrations(self, *directories):
        """Add migration directories to the container."""
        for directory in directories:
            self.app.bind(
                '{}_MigrationDirectory'.format(random_string(4)),
                directory
            )

    def commands(self, *commands):
        """Add commands to the container. Pass in the commands as arguments."""
        for command in commands:
            self.app.bind(
                '{}Command'.format(command.__class__.__name__.replace('Command', '')),
                command
            )

    def assets(self, assets):
        """Add assets to the container.

        Arguments:
            assets {dict} -- A dictionary of assets to add
        """
        self.app.make('Storage').STATICFILES.update(assets)



from .static import static
from .password import password
from .validator import validate
from .misc import random_string, dot, clean_request_input
from .Extendable import Extendable
from .time import cookie_expire_time



"""A PasswordController Module."""

import uuid

from masonite import env, Mail, Session
from masonite.auth import Auth
from masonite.helpers import password as bcrypt_password
from masonite.request import Request
from masonite.view import View

from config.auth import AUTH


class PasswordController:
    """Password Controller."""

    def forget(self, view: View, request: Request):
        return view.render('auth/forget', {'app': request.app().make('Application'), 'Auth': Auth(request)})

    def reset(self, request: Request):
        token = request.param('token')
        user = AUTH['model'].where('remember_token', token).first()
        if user:
            return view('auth/reset', {'token': token, 'app': request.app().make('Application'), 'Auth': Auth(request)})

    def send(self, request: Request, session: Session, mail: Mail):
        email = request.input('email')
        user = AUTH['model'].where('email', email).first()

        if user:
            if not user.remember_token:
                user.remember_token = str(uuid.uuid4())
                user.save()
            message = 'Please visit {}/password/{}/reset to reset your password'.format(env('SITE', 'http://localhost:8000'), user.remember_token)
            mail.subject('Reset Password Instructions').to(email).send(message)
            session.flash('success', 'Email sent. Follow the instruction in the email to reset your password.')
            return request.redirect('/password')
        else:
            session.flash('error', 'Could not send reset email. Please enter correct email.')
            return request.redirect('/password')

    def update(self, request: Request):
        user = AUTH['model'].where('remember_token', request.param('token')).first()
        if user:
            user.password = bcrypt_password(request.input('password'))
            user.save()
            return request.redirect('/login')




import time
import os

from hupper.logger import DefaultLogger, LogLevel
from hupper.reloader import Reloader, find_default_monitor_factory
from cleo import Command


class ServeCommand(Command):
    """
    Run the Masonite server.

    serve
        {--p|port=8000 : Specify which port to run the server}
        {--b|host=127.0.0.1 : Specify which ip address to run the server}
        {--r|reload : Make the server automatically reload on file changes}
        {--i|reload-interval=1 : Number of seconds to wait to reload after changed are detected}
    """

    def handle(self):
        if self.option('reload'):
            logger = DefaultLogger(LogLevel.INFO)

            # worker args are pickled and then passed to the new process
            worker_args = [
                self.option("host"), self.option("port"), "wsgi:application",
            ]

            reloader = Reloader(
                "masonite.commands._devserver.run",
                find_default_monitor_factory(logger),
                logger,
                worker_args=worker_args,
            )

            self._run_reloader(reloader, extra_files=[".env"])

        else:
            from wsgi import application
            from ._devserver import run
            run(self.option("host"), self.option("port"), application)

    def _run_reloader(self, reloader, extra_files=[]):
        reloader._capture_signals()
        reloader._start_monitor()
        for blob in extra_files:
            reloader.monitor.add_path(os.path.join(os.getcwd(), blob))
        try:
            while True:
                if not reloader._run_worker():
                    reloader._wait_for_changes()
                time.sleep(float(self.option('reload-interval')))
        except KeyboardInterrupt:
            pass
        finally:
            reloader._stop_monitor()
            reloader._restore_signals()



"""Module for miscellaneous helper methods."""

import random
import string


def random_string(length=4):
    """Generate a random string based on the length given.

    Keyword Arguments:
        length {int} -- The amount of the characters to generate (default: {4})

    Returns:
        string
    """
    return "".join(
        random.choice(string.ascii_uppercase + string.digits) for _ in range(length)
    )


def dot(data, compile_to=None):
    notation_list = data.split('.')

    compiling = ""
    compiling += notation_list[0]
    beginning_string = compile_to.split('{1}')[0]
    compiling = beginning_string + compiling
    dot_split = compile_to.replace(beginning_string + '{1}', '').split('{.}')
    if any(len(x) > 1 for x in dot_split):
        raise ValueError("Cannot have multiple values between {1} and {.}")

    for notation in notation_list[1:]:
        compiling += dot_split[0]
        compiling += notation
        compiling += dot_split[1]
    return compiling


def clean_request_input(value):
    import html

    if isinstance(value, str):
        return html.escape(value)
    elif isinstance(value, list):
        return [html.escape(x) for x in value]
    elif isinstance(value, int):
        return value
    elif isinstance(value, dict):
        return {key: html.escape(val) for (key, val) in value.items()}

    return value



"""Extendable Module."""

import inspect


class Extendable:
    """Add the ability to extend classes on the fly."""

    def extend(self, key, obj=None):
        """Extend the current class with an object.

        This essentially extends a class on the fly.

        Arguments:
            key {string} -- The name of the method you want to set

        Keyword Arguments:
            obj {object} -- Any function, method or class (default: {None})

        Returns:
            self
        """
        # If both key and an object is passed
        if obj:
            if inspect.ismethod(obj):
                obj = obj.__func__

            setattr(self, key, obj.__get__(self))
            return self

        # Extend all of a classes methods into this class
        if inspect.isclass(key):
            for method in inspect.getmembers(key, inspect.isfunction):
                setattr(self, method[0], method[1].__get__(self))

        # Extend a function into this class
        elif inspect.isfunction(key):
            setattr(self, key.__name__, key.__get__(self))
        elif inspect.ismethod(obj):
            setattr(self, key.__name__, key.__func__.__get__(self))
        return self



from abc import ABC, abstractmethod


class UploadContract(ABC):

    @abstractmethod
    def accept(self):
        pass

    @abstractmethod
    def validate_extension(self):
        pass

    @abstractmethod
    def store(self):
        pass



from .CsrfMiddleware import CsrfMiddleware
from .MaintenanceModeMiddleware import MaintenanceModeMiddleware
from .ResponseMiddleware import ResponseMiddleware
from .SecureHeadersMiddleware import SecureHeadersMiddleware



import requests
from config import application


class TestRequest:

    def __init__(self):
        self.request = None
        pass

    def get(self, route_url):
        self.request = requests.get(application.URL + route_url)
        return self

    def status_code(self, code):
        if self.request.status_code == code:
            return True

        return False

    def contains(self, content):
        if content in self.request.text:
            return True

        return False



"""New Validator Command."""
from masonite.commands import BaseScaffoldCommand


class ValidatorCommand(BaseScaffoldCommand):
    """
    Creates a validator.

    validator
        {name : Name of the validator}
    """

    scaffold_name = 'Validator'
    base_directory = 'app/validators/'
    template = '/masonite/snippets/scaffold/validator'



"""CSRF Protection Module."""
import binascii
import os


class Csrf:
    """CSRF Protection Class."""

    def __init__(self, request):
        """CSRF constructor.

        Arguments:
            request {masonite.request.Request} -- Request object
        """
        self.request = request

    def generate_csrf_token(self, length=30):
        """Generate CRSRF token.

        The // you see below is integer division. Since the token will be twice
        the size of the length passed to. A length of 30 passed below will generate
        a string length of 60 so we integer divide by 2

        Returns:
            string -- Returns token generated.
        """
        token = bytes(binascii.b2a_hex(os.urandom(length // 2))).decode('utf-8')
        self.request.cookie('csrf_token', token, encrypt=False)
        return token

    def verify_csrf_token(self, token):
        """Verify if csrf token is valid from the cookie set.

        Arguments:
            token {string} -- The token that was generated.

        Returns:
            bool
        """
        if self.request.get_cookie('csrf_token', decrypt=False) == token:
            return True
        else:
            return False



"""New Model Command."""
from masonite.commands import BaseScaffoldCommand


class ModelCommand(BaseScaffoldCommand):
    """
    Creates a model.

    model
        {name : Name of the model}
    """

    scaffold_name = "Model"
    template = '/masonite/snippets/scaffold/model'
    base_directory = 'app/'



"""SecureHeaders Middleware."""

from masonite.request import Request


class SecureHeadersMiddleware:
    """SecureHeaders Middleware."""

    def __init__(self, request: Request):
        """Inject Any Dependencies From The Service Container.

        Arguments:
            Request {masonite.request.Request} -- The Masonite request object
        """
        self.request = request
        self.headers = {
            'Strict-Transport-Security': 'max-age=63072000; includeSubdomains',
            'X-Frame-Options': 'SAMEORIGIN',
            'X-XSS-Protection': '1; mode=block',
            'X-Content-Type-Options': 'nosniff',
            'Referrer-Policy': 'no-referrer, strict-origin-when-cross-origin',
            'Cache-control': 'no-cache, no-store, must-revalidate',
            'Pragma': 'no-cache',
        }

    def before(self):
        """Run This Middleware Before The Route Executes.
        """
        pass

    def after(self):
        """Run This Middleware After The Route Executes.
        """
        from config import middleware

        try:
            # Try importing secure headers if they exist in the config file
            self.headers.update(middleware.SECURE_HEADERS)
        except AttributeError:
            pass

        self.request.header(self.headers)



"""Autoloader Module.

This contains the class for autoloading classes from directories.
This class is simply used to point at a directory and retrieve all classes in that directory.
"""

import inspect
import pkgutil
from pydoc import importlib

from masonite.exceptions import (AutoloadContainerOverwrite, ContainerError,
                                 InvalidAutoloadPath)


class Autoload:
    """Autoload class. Used to retrieve all classes from any set of directories."""

    classes = {}

    def __init__(self, app=None):
        """Autoload Constructor.

        Keyword Arguments:
            app {masonite.app.App} -- Container class (default: {None})
        """
        self.app = app

    def load(self, directories, instantiate=False):
        """Load all classes found in a list of directories into the container.

        Arguments:
            directories {list} -- List of directories to search.

        Keyword Arguments:
            instantiate {bool} -- Whether or not to instantiate the class (default: {False})

        Raises:
            ContainerError -- Thrown when the container is not loaded into the class.
            AutoloadContainerOverwrite -- Thrown when the container already has the key binding.
        """
        self.instantiate = instantiate
        if not self.app:
            raise ContainerError(
                'Container not specified. Pass the container into the constructor')

        for (module_loader, name, ispkg) in pkgutil.iter_modules(directories):
            search_path = module_loader.path
            for obj in inspect.getmembers(self._get_module_members(module_loader, name)):

                # If the object is a class and the objects module starts with the search path
                if inspect.isclass(obj[1]) and obj[1].__module__.split('.')[:-1] == search_path.split('/'):
                    if self.app.has(obj[1].__name__) and not self.app.make(obj[1].__name__).__module__.startswith(search_path):
                        raise AutoloadContainerOverwrite(
                            'Container already has the key: {}. Cannot overwrite a container key that exists outside of your application.'.format(obj[1].__name__))
                    self.app.bind(obj[1].__name__, self._can_instantiate(obj))

    def instances(self, directories, instance, only_app=True, instantiate=False):
        """Use to autoload all instances of a specific object.

        Arguments:
            directories {list} -- List of directories to search.
            instance {object} -- Object to search for instances of.

        Keyword Arguments:
            only_app {bool} -- Only search in the current application namespace. This will not found other classes
                               that are imported through third party packages. (default: {True})
            instantiate {bool} -- Whether or not to instantiate the classes it finds. (default: {False})

        Returns:
            dict -- Returns a dictionary of classes it found.
        """
        self.instantiate = instantiate

        for (module_loader, name, ispkg) in pkgutil.iter_modules(directories):
            search_path = module_loader.path
            for obj in inspect.getmembers(self._get_module_members(module_loader, name)):
                if inspect.isclass(obj[1]) and issubclass(obj[1], instance):
                    if only_app and obj[1].__module__.startswith(search_path.replace('/', '.')):
                        self.classes.update(
                            {obj[1].__name__: self._can_instantiate(obj)})
                    elif not only_app:
                        self.classes.update(
                            {obj[1].__name__: self._can_instantiate(obj)})

        return self.classes

    def collect(self, directories, only_app=True, instantiate=False):
        """Collect all classes from a specific list of directories.

        Arguments:
            directories {list} -- List of directories to search.

        Keyword Arguments:
            only_app {bool} -- Only search in the current application namespace. This will not found other classes
                               that are imported through third party packages. (default: {True})
            instantiate {bool} -- Whether or not to instantiate the classes it finds. (default: {False})

        Returns:
            dict -- Returns a dictionary of objects found and their key bindings.
        """
        self.instantiate = instantiate

        for (module_loader, name, ispkg) in pkgutil.iter_modules(directories):
            search_path = module_loader.path

            for obj in inspect.getmembers(self._get_module_members(module_loader, name)):
                if inspect.isclass(obj[1]):
                    if only_app and obj[1].__module__.startswith(search_path.replace('/', '.')):
                        self.classes.update(
                            {obj[1].__name__: self._can_instantiate(obj)})
                    elif not only_app:
                        self.classes.update(
                            {obj[1].__name__: self._can_instantiate(obj)})

        return self.classes

    def _can_instantiate(self, obj):
        """Instantiate the class or not depending on the property set.

        Arguments:
            obj {object} -- Object to check for instantiation.

        Returns:
            object -- Returns the object being instantiated.
        """
        if self.instantiate:
            return obj[1]()

        return obj[1]

    def _get_module_members(self, module_loader, name):
        """Get the module members.

        Arguments:
            module_loader {pkgutil.ModuleLoader} -- Module Loader from the pkgutil library
            name {string} -- Name of the module

        Raises:
            InvalidAutoloadPath -- Thrown when the search path ends with a forward

        Returns:
            module -- returns the imported module.
        """
        search_path = module_loader.path
        if search_path.endswith('/'):
            raise InvalidAutoloadPath(
                'Autoload path cannot have a trailing slash')

        return importlib.import_module(
            module_loader.path.replace('/', '.') + '.' + name)



"""Verify Email Module."""

import time

from masonite.auth.Sign import Sign


class MustVerifyEmail:
    """Class To Verify User Email."""

    def verify_email(self, mail_manager, request):
        mail = mail_manager.helper()
        sign = Sign()

        token = sign.sign('{0}::{1}'.format(self.id, time.time()))
        link = '{0}/email/verify/{1}'.format(request.environ['HTTP_HOST'], token)

        mail.to(self.email) \
            .template('auth/verifymail', {'name': self.name, 'email': self.email, 'link': link}) \
            .subject('Please Confirm Your Email').send()



from abc import ABC, abstractmethod


class CacheContract(ABC):

    @abstractmethod
    def store(self):
        pass

    @abstractmethod
    def store_for(self):
        pass

    @abstractmethod
    def get(self):
        pass

    @abstractmethod
    def is_valid(self):
        pass

    @abstractmethod
    def exists(self):
        pass



"""Session Memory Module."""

from masonite.contracts import SessionContract
from masonite.drivers import BaseDriver
from masonite.app import App


class SessionMemoryDriver(SessionContract, BaseDriver):
    """Memory Session Driver."""

    _session = {}
    _flash = {}

    def __init__(self, app: App):
        """Cookie Session Constructor.

        Arguments:
            Environ {dict} -- The WSGI environment
        """
        self.environ = app.make('Environ')

    def get(self, key):
        """Get a value from the session.

        Arguments:
            key {string} -- The key to get from the session.

        Returns:
            string|None - Returns None if a value does not exist.
        """
        data = self.__collect_data(key)
        if data:
            return data

        return None

    def set(self, key, value):
        """Set a vlue in the session.

        Arguments:
            key {string} -- The key to set as the session key.
            value {string} -- The value to set in the session.
        """
        ip = self.__get_client_address()

        if ip not in self._session:
            self._session[ip] = {}

        self._session[ip][key] = value

    def has(self, key):
        """Check if a key exists in the session.

        Arguments:
            key {string} -- The key to check for in the session.

        Returns:
            bool
        """
        data = self.__collect_data()
        if data and key in data:
            return True
        return False

    def all(self):
        """Get all session data.

        Returns:
            dict
        """
        return self.__collect_data()

    def flash(self, key, value):
        """Add temporary data to the session.

        Arguments:
            key {string} -- The key to set as the session key.
            value {string} -- The value to set in the session.
        """
        ip = self.__get_client_address()
        if ip not in self._flash:
            self._flash[ip] = {}

        self._flash[ip][key] = value

    def reset(self, flash_only=False):
        """Delete all session data.

        Keyword Arguments:
            flash_only {bool} -- If only flash data should be deleted. (default: {False})
        """
        ip = self.__get_client_address()

        if flash_only:
            if ip in self._flash:
                self._flash[ip] = {}
        else:
            if ip in self._session:
                self._session[ip] = {}

    def delete(self, key):
        """Delete a value in the session by it's key.

        Arguments:
            key {string} -- The key to find in the session.

        Returns:
            bool -- If the key was deleted or not
        """
        data = self.__collect_data()

        if key in data:
            del data[key]
            return True

        return False

    def __get_client_address(self):
        """Get ip from the client."""
        if 'HTTP_X_FORWARDED_FOR' in self.environ:
            return self.environ['HTTP_X_FORWARDED_FOR'].split(',')[-1].strip()

        return self.environ['REMOTE_ADDR']

    def __collect_data(self, key=False):
        """Collect data from session and flash data.

        Returns:
            dict
        """
        ip = self.__get_client_address()

        # Declare a new dictionary
        session = {}

        # If the session data has keys
        if ip in self._session:
            session = self._session[ip]

        # If the session flash has keys
        if ip in self._flash:
            session.update(self._flash[ip])

        # If a key is set and it is inside the new declared session, return that key
        if key and key in session:
            return session[key]

        # If the key is set and is not in the session
        if key and key not in session:
            return None

        # If the session is still an empty dictionary
        if not session:
            return None

        # No checks have been hit. Return the new dictionary
        return session

    def helper(self):
        """Used to create builtin helper function."""
        return self



"""Queue Manager Module."""

from masonite.contracts import QueueManagerContract
from masonite.managers import Manager


class QueueManager(Manager, QueueManagerContract):
    """Manages all queue drivers.

    Arguments:
        Manager {masonite.managers.Manager} -- The base Manager class.
    """

    config = 'QueueConfig'
    driver_prefix = 'Queue'


class Queue:
    """Dummy class that will be used to swap out the manager in the container."""

    pass



import json

from masonite.app import App
from masonite.exceptions import ResponseError
from masonite.request import Request
from masonite.response import Response


class ResponseMiddleware:

    def __init__(self, request: Request, app: App, response: Response):
        self.request = request
        self.app = app
        self.response = response

    def after(self):
        if self.request.redirect_url:
            self.response.redirect(self.request.redirect_url, status=302)
            self.request.reset_redirections()

        if self.app.has('Session') and self.request.is_status(200):
            self.app.make('Session').reset(flash_only=True)



from .BroadcastContract import BroadcastContract
from .CacheContract import CacheContract
from .MailContract import MailContract
from .QueueContract import QueueContract
from .SessionContract import SessionContract
from .UploadContract import UploadContract

from .managers.BroadcastManagerContract import BroadcastManagerContract
from .managers.CacheManagerContract import CacheManagerContract
from .managers.MailManagerContract import MailManagerContract
from .managers.QueueManagerContract import QueueManagerContract
from .managers.SessionManagerContract import SessionManagerContract
from .managers.UploadManagerContract import UploadManagerContract
